API_VERSION = "v2"
API_ENDPOINT_PATH = f"/portal/api/external/{API_VERSION}"

LOGIN_CREDENTIAL = "/core/auth/create-access-token-by-credentials"
LOGIN_SERVICE = "/core/auth/create-access-token-by-service-account"
